@@ -92,7 +92,7 @@ class ProfileStorage:
     fact that there is a temporary directory are both
     implementation details and not exposed in the API.
 
-    Profile files are stored in /etc/desktop-profiles.
+    Profile files are stored in /etc/sabayon/profiles.
 
     Each file or directory in the profile has metadata
     associated with it - the "source" of the file/directory
@@ -104,7 +104,7 @@ class ProfileStorage:
         """Create a ProfileStorage.
 
         @name: the name of the profile - translates to
-        /etc/desktop-profiles/$(name).zip.
+        /etc/sabayon/profiles/$(name).zip.
         """
         self.name = name
         self.readonly = 0
@@ -151,6 +151,7 @@ class ProfileStorage:
         metadata = libxml2.newDoc ("1.0")
         root = metadata.newChild (None, "metadata", None)
         root.newChild (None, "directories", None)
+        root.newChild (None, "links", None)
         root.newChild (None, "files", None)
         return metadata
 
@@ -204,14 +205,15 @@ class ProfileStorage:
             attributes[attrnode.prop ("name")]  = attrnode.prop ("value")
         return attributes
 
-    def __update_file_or_dir_node (self, file_or_dir_node, source, attributes, metadata):
+    def __update_file_or_dir_node (self, file_or_dir_node, statinfo, source, attributes, metadata):
         # Set properties
-
         unlink_children (file_or_dir_node)
 
         file_or_dir_node.setProp ("timestamp", str (int (time.time ())))
         file_or_dir_node.setProp ("user",      util.get_user_name ())
         file_or_dir_node.setProp ("host",      socket.gethostname ())
+        file_or_dir_node.setProp ("st_mode",   str (statinfo.st_mode))
+        file_or_dir_node.setProp ("st_mtime",  str (statinfo.st_mtime))
 
         # Set source and attributes
         file_or_dir_node.newChild (None, "source", source)
@@ -223,7 +225,7 @@ class ProfileStorage:
                 attribute_node.setProp ("name",  str (name))
                 attribute_node.setProp ("value", str (attributes[name]))
 
-    def __update_file_node (self, path, source, attributes, metadata = None):
+    def __update_file_node (self, path, statinfo, source, attributes, metadata = None):
         if metadata is None:
             metadata = self.metadata
         assert metadata
@@ -238,9 +240,9 @@ class ProfileStorage:
             file_node = files_node.newChild (None, "file", None)
             file_node.setProp ("path", path)
 
-        self.__update_file_or_dir_node (file_node, source, attributes, metadata)
+        self.__update_file_or_dir_node (file_node, statinfo, source, attributes, metadata)
 
-    def __update_directory_node (self, path, source, attributes, metadata = None):
+    def __update_directory_node (self, path, statinfo, source, attributes, metadata = None):
         if metadata is None:
             metadata = self.metadata
         assert metadata
@@ -255,7 +257,25 @@ class ProfileStorage:
             directory_node = files_node.newChild (None, "directory", None)
             directory_node.setProp ("path", path)
 
-        self.__update_file_or_dir_node (directory_node, source, attributes, metadata)
+        self.__update_file_or_dir_node (directory_node, statinfo, source, attributes, metadata)
+
+    def __update_link_node (self, path, statinfo, source, attributes, metadata = None):
+        if metadata is None:
+            metadata = self.metadata
+        assert metadata
+
+        files_node = metadata.xpathEval ("/metadata/links")[0]
+
+        # Ensure the directory node exists
+        nodes = files_node.xpathEval ("link[@path='%s']" % path)
+        if len (nodes):
+            link_node = nodes[0]
+        else:
+            link_node = files_node.newChild (None, "link", None)
+            link_node.setProp ("path", path)
+            link_node.setProp ("dest", os.readlink(path))
+
+        self.__update_file_or_dir_node (link_node, statinfo, source, attributes, metadata)
 
     def __unpack (self):
         self.__read_metadata ()
@@ -283,7 +303,6 @@ class ProfileStorage:
                 if not os.path.exists (dest_dir):
                     os.makedirs (dest_dir)
 
-                # It sucks that we lose file permissions, mtime etc. with ZIP
                 zip.extract(f, dir)
 
         def unzip_foreach (path, is_directory, data):
@@ -301,7 +320,6 @@ class ProfileStorage:
                 if not os.path.exists (dest_dir):
                     os.makedirs (dest_dir)
 
-                # It sucks that we lose file permissions, mtime etc. with ZIP
                 zip.extract(path, temp_path)
 
         self.__foreach_all (unzip_foreach, (self.zip, self.temp_path))
@@ -357,9 +375,13 @@ class ProfileStorage:
             src_path = path
         src_path = os.path.join (src_dir, src_path)
         dst_path = os.path.join (self.temp_path, path)
+        statinfo = os.lstat(src_path)
 
         if not os.path.exists (src_path):
-            raise ProfileStorageException (_("Cannot add non-existent file '%s'") % src_path)
+            # change the raise to a dprint, since some files seem to disappear.
+            # raise ProfileStorageException (_("Cannot add non-existent file '%s'") % src_path)
+            dprint (_("Cannot add non-existent file '%s'"), src_path)
+            return
 
         # Remove old version
         node = self.__get_node (path)
@@ -369,10 +391,12 @@ class ProfileStorage:
         recursive_del (dst_path)
 
         if os.path.isdir (src_path):
-            self.__update_directory_node (path, source, attributes)
+            self.__update_directory_node (path, statinfo, source, attributes)
             copy_tree (self.temp_path, src_dir, path)
-        else:
-            self.__update_file_node (path, source, attributes)
+        elif os.path.islink (src_path):
+            self.__update_link_node (path, statinfo, source, attributes)
+        elif os.path.isfile (src_path) and not os.path.islink (src_path):
+            self.__update_file_node (path, statinfo, source, attributes)
             dirname = os.path.dirname (dst_path)
             if not os.path.exists (dirname):
                 os.makedirs (dirname)
@@ -389,6 +413,15 @@ class ProfileStorage:
             return dir_nodes[0]
         return None
 
+    def __get_link_node (self, path, metadata = None):
+        if metadata is None:
+            metadata = self.metadata
+        assert metadata
+        link_nodes = metadata.xpathEval ("/metadata/links/link[@path='%s']" % path)
+        if len (link_nodes) > 0:
+            return link_nodes[0]
+        return None
+
     def __get_file_node (self, path, metadata = None):
         if metadata is None:
             metadata = self.metadata
@@ -402,6 +435,9 @@ class ProfileStorage:
         self.__read_metadata ()
 
         node = self.__get_dir_node (path, metadata)
+        if node:
+            return node
+        node = self.__get_link_node (path, metadata)
         if node:
             return node
         node = self.__get_file_node (path, metadata)
@@ -436,6 +472,9 @@ class ProfileStorage:
         node = self.__get_dir_node (path)
         if node:
             return "directory"
+        node = self.__get_link_node (path)
+        if node:
+            return "link"
         node = self.__get_file_node (path)
         if node:
             return "file"
@@ -481,12 +520,17 @@ class ProfileStorage:
                     raise err
 
             if got_stat:
-                try:
-                    os.unlink (dest)
-                except OSError, err:
-                    raise ProfileStorageException (_("Couldn't unlink file '%s'") % dest)
+                if os.path.isdir (dest):
+                    try:
+                        os.rmdir (dest)
+                    except OSError, err:
+                        raise ProfileStorageException (_("Couldn't rmdir '%s'") % dest)
+                else:
+                    try:
+                        os.unlink (dest)
+                    except OSError, err:
+                        raise ProfileStorageException (_("Couldn't unlink file '%s'") % dest)
 
-            # FIXME: we lose the "original" permissions, mtime, etc. with ZIP files.
             shutil.copy2 (src, dest)
 
             if got_stat:
@@ -500,6 +544,14 @@ class ProfileStorage:
 
         if item_type == "directory":
             copy_tree (dst_dir, self.temp_path, path, None, overwrite)
+            dir_node = self.__get_dir_node (path)
+            st_mode = dir_node.prop ("st_mode")
+            if st_mode:
+                os.chmod (os.path.join(dst_dir,path), int (st_mode))
+        elif item_type == "link":
+            link_node = self.__get_link_node (path)
+            dest = link_node.prop("dest")
+            os.symlink(dest, path)
         else:
             dst_path = os.path.join (dst_dir, path)
             if overwrite or not os.path.exists (dst_path):
@@ -508,6 +560,10 @@ class ProfileStorage:
                     os.makedirs (dirname)
 
                 copy_preserving_permissions (os.path.join (self.temp_path, path), dst_path)
+                file_node = self.__get_file_node (path)
+                st_mode = file_node.prop ("st_mode")
+                if st_mode:
+                    os.chmod (dst_path, int (st_mode))
 
     def list (self, source = None):
         """List the current contents of the profile.
@@ -556,15 +612,18 @@ class ProfileStorage:
         """
         self.__read_metadata ()
 
-        for node in self.metadata.xpathEval ("/metadata/files/file"):
+        for node in self.metadata.xpathEval ("/metadata/directories/directory"):
             self.__foreach_node (node, callback, user_data, source)
 
-        for node in self.metadata.xpathEval ("/metadata/directories/directory"):
+        for node in self.metadata.xpathEval ("/metadata/links/link"):
+            self.__foreach_node (node, callback, user_data, source)
+
+        for node in self.metadata.xpathEval ("/metadata/files/file"):
             self.__foreach_node (node, callback, user_data, source)
 
     def save (self):
         """Save the contents of the profile to
-        /etc/desktop-profiles/$(name).zip.
+        /etc/sabayon/profiles/$(name).zip.
         """
         if self.readonly:
             raise ProfileStorageException (_("Profile is read-only %s") %
@@ -602,7 +661,7 @@ class ProfileStorage:
                     if os.path.isdir (path):
                         # We need to stop if we are recursing inside a ignored 
                         # directory.
-                        if util.should_ignore_dir (homedir, 
+                        if util.should_ignore_dir (homedir,
                                                    DIRECTORIES_TO_IGNORE_PROFILE,
                                                    os.path.join (homedir, name, f)):
                             dprint ("Not going inside %s as it is an ignored directory.", path)
@@ -610,12 +669,12 @@ class ProfileStorage:
                             zip_directory (save_zip,
                                         path,
                                         os.path.join (name, f))
-                    elif os.path.isfile (path):
+                    elif os.path.isfile (path) and not os.path.islink (path):
                         # Avoid putting in a duplicate file entry
                         # See bug #476761
                         if os.path.join (name, f) in zip_filelist:
                             dprint ("Not adding %s to zipfile since it is already in the file", os.path.join (name, f))
-                        elif util.should_ignore_file (homedir, 
+                        elif util.should_ignore_file (homedir,
                                                       DIRECTORIES_TO_IGNORE_PROFILE,
                                                       FILES_TO_IGNORE,
                                                       os.path.join (homedir, name, f)):
